from dataclasses import make_dataclass, field
from typing import get_type_hints, Optional


def entity(table: str):
    def _entity(cls):
        annotations = get_type_hints(cls)
        new_fields = []

        for name, typ in annotations.items():
            if name.startswith('__'):
                continue
            # 将所有字段默认值设为 None，类型设为 Optional[原类型]
            new_fields.append((name, Optional[typ], field(default=None)))

        new_cls = make_dataclass(cls.__name__, new_fields, bases=cls.__bases__)
        new_cls.__table_name__ = table

        for new_field in new_fields:
            setattr(new_cls, new_field[0], new_field[0])

        return new_cls

    return _entity


def vo(cls):
    annotations = get_type_hints(cls)
    new_fields = []

    for name, typ in annotations.items():
        if name.startswith('__'):
            continue
        # 将所有字段默认值设为 None，类型设为 Optional[原类型]
        new_fields.append((name, Optional[typ], field(default=None)))

    new_cls = make_dataclass(cls.__name__, new_fields, bases=cls.__bases__)
    return new_cls



from . import dorm
from ._context import dorm_context
from .protocols import IDataSource


def begin(data_source: IDataSource | None = None):
    dorm_context.get().begin(data_source or dorm.default_datasource())


def commit():
    dorm_context.get().commit()


def rollback():
    dorm_context.get().rollback()



import uuid
from contextvars import ContextVar

from loguru import logger
from pymysql.connections import Connection as MysqlConnection

from .protocols import IDataSource


class DormContext:
    def __init__(self):
        self._conn: MysqlConnection | None = None
        self._tx_id = None

    def begin(self, ds: IDataSource):
        self._tx_id = uuid.uuid4()
        self._conn = ds.get_connection()
        self._conn.begin()

        logger.debug(f'begin transaction: {self._tx_id}')

    def commit(self):
        self._conn.commit()
        logger.debug(f'commit transaction: {self._tx_id}')
        self._tx_id = None
        self._conn = None

    def rollback(self):
        self._conn.rollback()
        logger.debug(f'rollback transaction: {self._tx_id}')
        self._tx_id = None
        self._conn = None

    def tx(self):
        return self._conn

    def tx_id(self):
        return self._tx_id


dorm_context = ContextVar('sql_context', default=DormContext())



def build_select(query_wrapper) -> tuple[str, tuple[any, ...]]:
    sql = f'SELECT {",".join(query_wrapper.field_list)} FROM {query_wrapper.table}'
    args = ()
    if len(query_wrapper.condition_tree.conditions) > 0:
        exp, args = query_wrapper.condition_tree.parse()
        sql += ' WHERE ' + exp
    if query_wrapper.order_by is not None:
        sql += f' ORDER BY {",".join(query_wrapper.order_by)}'
    if query_wrapper.limit_ is not None:
        sql += f' LIMIT {query_wrapper.limit_}'
    if query_wrapper.offset is not None:
        sql += f' OFFSET {query_wrapper.offset}'

    return sql, args
